#!/usr/bin/perl

# nessrpts-v0.05.pl by Randy Nash (nashr@atriskonline.com)
# Copyright (C) 2002-2005 Randy Nash - @RISK Online (www.atriskonline.com)
#
# nessrpts is part of the nessQuick package which can be downloaded at
# http://www.atriskonline.com/projects/nessQuick.shtml
#
# nessrpt: Generates a series of reports from Nessus .nbe data.  Reports are:
#   1. Total count of all findings
#   2. List of all Hosts that were scanned
#   3. Hosts with Operating System guess
#   4. Risk Counts by IP
#   5. Vulnerabilities with affected hosts
#
# --- GNU General Public License Disclamer ---
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
use strict;
# use warnings;
# use diagnostics;

use DBI;

# Connect To Database
#
my $database = "nessquick";
my $hostname = "localhost";
my $username = "root";
my $password = "";
my $dbh = DBI->connect("DBI:mysql:$database:$hostname", $username, $password);
die "Cannot log into database.  Please try again later.\n" unless $dbh;

# This section on hold for now...
# Name of report file
#
#print "Enter desired report filename: ";
#$outfile = <STDIN>;
#chomp $outfile;
#
#open the output file 
#open(REPORT, ">$outfile") || die("Couldn't open report file"); 

# Risk Values
my %RISK_VALUES =
  (0 => "Informational Notes",
   1 => "Security Notes",
   2 => "Security Warnings",
   3 => "Security Holes"
  );

##################################################
# Report 1: Total count of all findings
# This serves as an Executive Summary of findings
#
my $sth1 = $dbh->prepare("SELECT riskval, count(riskval)
                          FROM results
                          GROUP BY -(riskval)")
	or die "Can't prepare: $dbh->errstr\n";

# Execute query
#
my $rv1 = $sth1->execute
	or die "can't execute the query: $sth1->errstr";

# "while" loop to produce report
#
print "\nNessus found the following:\n\n";

while (my @array1 = $sth1->fetchrow_array)
	{
    (my $riskval, my $count) = @array1;
    my $risklevel = $RISK_VALUES{$riskval};
    print "\t$count\t$risklevel\n";
  }
print "\f";

##################################################
# Report 2: List of all Hosts that were scanned
#
my $sth2 = $dbh->prepare("SELECT host
                          FROM results
                          GROUP BY host")
	or die "Can't prepare: $dbh->errstr\n";

# Execute query
#
my $rv2 = $sth2->execute
	or die "can't execute the query: $sth2->errstr";

# "while" loop to produce report
#
print "\nNessus scanned the following hosts:\n\n";

while (my @array2 = $sth2->fetchrow_array)
	{
    (my $host) = @array2;
    print "\t$host\n";
  }
print "\f";

##################################################
# Report 3: Hosts with Operating System guess
#
my $sth3 = $dbh->prepare("SELECT scriptid, host, msg
                          FROM results
                          WHERE scriptid = 11936
                          ORDER BY msg")
	or die "Can't prepare: $dbh->errstr\n";

# Execute query
#
my $rv3 = $sth3->execute
	or die "can't execute the query: $sth3->errstr";

# "while" loop to produce report
#
print "\nNessus assumes the following Operating Systems are in use:\n\n";

while (my @array3 = $sth3->fetchrow_array)
	{
    (my $scriptid, my $host, my $msg) = @array3;
    print "$host\n$msg\n";
  }
print "\f";

##################################################
# Report 4: Risk Counts by IP
#
my $sth4 = $dbh->prepare("SELECT host, riskval, count(riskval)
                          FROM results
                          GROUP BY host, -(riskval)")
	or die "Can't prepare: $dbh->errstr\n";

# Execute query
#
my $rv4 = $sth4->execute
	or die "can't execute the query: $sth4->errstr";

print "\nCount of findings by IP/Host:\n\n";

my $first4 = "yes";
my $lasthost = " ";

# "while" loop to produce report
#
while (my @array4 = $sth4->fetchrow_array)
	{
    (my $host, my $riskval, my $count) = @array4;

      if ($host ne $lasthost)
        {
          if ($first4 eq "yes")
            {
              print "\n\t$host";
              $first4 = "no";
              $lasthost = $host;
            }
          else
            {
#              print "\t\t";
              $first4 = "yes";
              $lasthost = " ";
            }
        }
      else
        {
          print "\t\t";
          $first4 = "yes";
          $lasthost = $host;
        }
    my $risklevel = $RISK_VALUES{$riskval};
    print "\t$count\t$risklevel\n";
  }
print "\f";

##################################################
# Report 5: Vulnerabilities with affected hosts
#
my $sth5 = $dbh->prepare("SELECT riskval, scriptid, service, host, msg
                          FROM results
                          WHERE riskval != 0
                          ORDER by -riskval, scriptid, service, host")
or die "Can't prepare: $dbh->errstr\n";

# Execute query
#
my $rv5 = $sth5->execute
	or die "can't execute the query: $sth5->errstr";

print "\nVulnerabilities (by severity) with all affected hosts:\n\n";

my $lastmatch = " ";

# "while" loop to produce report
#
while (my @array5 = $sth5->fetchrow_array)
	{
    (my $riskval, my $scriptid, my $service, my $host, my $msg) = @array5;
    if ($scriptid ne $lastmatch)
      {
        print "\n********************************************************************************\n\n";
        print "$riskval\t$scriptid\t$service\n$msg\n";
        print "Affected hosts:\n\n";
        $lastmatch = $scriptid;
      }
    else
      {
        $lastmatch = $scriptid;
      }
    print "\t$host\n";
  }
print "\f";

$dbh->disconnect;

exit(0); 

